Texture2D texture2d <string uiname="Texture";>;

//This buffer has 32bit integers, every integer is in fact two 16bit depth values.
StructuredBuffer<int> Depth;

//These are just plain XY coordinates of the grid.
StructuredBuffer<float2> XY;

float Count;
float DepthScale;
float DepthScaleAdjustment;
bool ClipPointsAtZero;

SamplerState linearSampler : IMMUTABLE
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
};
 
cbuffer cbPerDraw : register( b0 )
{
	float4x4 tVP : LAYERVIEWPROJECTION;
};

cbuffer cbPerObj : register( b1 )
{
	float4 cAmb <bool color=true;String uiname="Color";> = { 1.0f,1.0f,1.0f,1.0f };
	float4x4 tW : WORLD;
};

struct VS_IN
{
	uint ii : SV_InstanceID;
	float4 PosO : POSITION;
	float4 TexCd : TEXCOORD0;

};

struct vs2ps
{
    float4 PosWVP: SV_Position;
	float4 Depth: TEXCOORD0;
    float4 TexCd: TEXCOORD1;
};

vs2ps VS(VS_IN input)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
	
	int depth;
	
	//One point gets one part of the 32bit integer
	if ( fmod (input.ii, 2) == 0)
	{
		depth = Depth[input.ii/2] & 0x0000FFFF;
		
	}
	//Next point gets the second part of The Same integer
	else
	{
		depth = Depth[input.ii/2] >> 16;
	}
	
	//4D vector is constructed from the XY and Depth values
	float4 myPoint = float4 (XY[input.ii], depth*DepthScale*DepthScaleAdjustment, 1.0f);
	
	myPoint = mul(mul(myPoint, tW), tVP);
    Out.PosWVP  = input.PosO + myPoint;
	
	//Sending Depth info also to the Pixelshader to perform clipping there.
	Out.Depth = float4(depth, 0, 0 , 0);
    return Out;
}




float4 PS(vs2ps In): SV_Target
{
	// Clipping points at 0
	if (ClipPointsAtZero)
	{
		clip (In.Depth.x > 0 ? 1 : -1);	
	}
	
	float4 col = texture2d.Sample(linearSampler,In.TexCd.xy) * cAmb;
    return col;
}





technique10 Constant
{
	pass P0
	{
		SetVertexShader( CompileShader( vs_4_0, VS() ) );
		SetPixelShader( CompileShader( ps_4_0, PS() ) );
	}
}
